import type { ArkErrors } from "@ark/schema";
import type { inferred } from "@ark/util";
import type { GenericHktParser } from "../generic.ts";
import type { Module } from "../module.ts";
import { type Scope } from "../scope.ts";
import type { DeclarationParser, DefinitionParser, TypeParser } from "../type.ts";
import { arkBuiltin } from "./builtin.ts";
import { arkFormat } from "./format.ts";
import { arkJs } from "./js.ts";
import { arkNumber } from "./number.ts";
import { arkParse } from "./parse.ts";
import { arkPlatform } from "./platform.ts";
import { arkString } from "./string.ts";
import { arkTs } from "./ts.ts";
import { arkTypedArray } from "./typedArray.ts";
export interface Ark extends Omit<Ark.keywords, keyof Ark.Wrapped>, Ark.Wrapped {
    TypedArray: arkTypedArray.submodule;
    parse: arkParse.submodule;
    format: arkFormat.submodule;
}
export declare namespace Ark {
    interface keywords extends arkTs.keywords, arkJs.keywords, arkPlatform.keywords, arkString.keywords, arkNumber.keywords, arkBuiltin.keywords {
    }
    interface Wrapped {
        string: arkString.submodule;
        number: arkNumber.submodule;
    }
}
export declare const ambient: Scope<Ark>;
export declare const ark: Module<Ark>;
export declare const type: TypeParser<{}>;
export declare namespace type {
    type cast<t> = {
        [inferred]?: t;
    };
    type errors = ArkErrors;
}
export declare const generic: GenericHktParser<{}>;
export declare const define: DefinitionParser<{}>;
export declare const declare: DeclarationParser<{}>;
