import type { ArkErrors, BaseRoot, Disjoint, JsonSchema, MetaSchema, Morph, UndeclaredKeyBehavior } from "@ark/schema";
import type { anyOrNever, Callable, ErrorMessage, inferred, Json, unset } from "@ark/util";
import type { distillConstrainableIn, distillConstrainableOut, distillIn, distillOut, distillValidatedOut, inferMorphOut, inferPipes } from "../ast.ts";
import type { inferIntersection } from "../intersect.ts";
import type { Scope } from "../scope.ts";
import type { inferTypeRoot, validateTypeRoot } from "../type.ts";
import type { ArrayType } from "./array.ts";
import type { instantiateType } from "./instantiate.ts";
/** @ts-ignore cast variance */
interface Type<out t = unknown, $ = {}> extends Callable<(data: unknown) => distillOut<t> | ArkErrors> {
    t: t;
    tIn: distillConstrainableIn<t>;
    tOut: distillConstrainableOut<t>;
    tValidatedOut: distillValidatedOut<t>;
    infer: distillOut<t>;
    inferIn: distillIn<t>;
    [inferred]: t;
    json: Json;
    toJsonSchema(): JsonSchema;
    description: string;
    expression: string;
    internal: BaseRoot;
    $: Scope<$>;
    assert(data: unknown): this["infer"];
    allows(data: unknown): data is this["inferIn"];
    traverse(data: unknown): distillOut<t> | ArkErrors;
    configure(meta: MetaSchema): this;
    describe(description: string): this;
    onUndeclaredKey(behavior: UndeclaredKeyBehavior): this;
    onDeepUndeclaredKey(behavior: UndeclaredKeyBehavior): this;
    from(literal: this["inferIn"]): this["infer"];
    as<t = unset>(...args: validateChainedAsArgs<t>): instantiateType<t, $>;
    get in(): instantiateType<this["tIn"], $>;
    get out(): instantiateType<this["tValidatedOut"], $>;
    intersect<const def, r = inferTypeRoot<def, $>>(def: validateTypeRoot<def, $>): instantiateType<inferIntersection<t, r>, $> | Disjoint;
    and<const def, r = inferTypeRoot<def, $>>(def: validateTypeRoot<def, $>): instantiateType<inferIntersection<t, r>, $>;
    or<const def, r = inferTypeRoot<def, $>>(def: validateTypeRoot<def, $>): instantiateType<t | r, $>;
    array(): ArrayType<t[], $>;
    pipe: ChainedPipes<t, $>;
    equals<def>(def: validateTypeRoot<def, $>): this is instantiateType<inferTypeRoot<def, $>, $>;
    extract<def>(r: validateTypeRoot<def, $>): instantiateType<Extract<t, inferTypeRoot<def, $>>, $>;
    exclude<def>(r: validateTypeRoot<def, $>): instantiateType<Exclude<t, inferTypeRoot<def, $>>, $>;
    extends<def>(other: validateTypeRoot<def, $>): this is instantiateType<inferTypeRoot<def, $>, $>;
    overlaps<def>(r: validateTypeRoot<def, $>): boolean;
    /** @deprecated */
    apply: Function["apply"];
    /** @deprecated */
    bind: Function["bind"];
    /** @deprecated */
    call: Function["call"];
    /** @deprecated */
    caller: Function;
    /** @deprecated */
    length: number;
    /** @deprecated */
    name: string;
    /** @deprecated */
    prototype: Function["prototype"];
    /** @deprecated */
    arguments: Function["arguments"];
    /** @deprecated */
    Symbol: never;
}
interface ChainedPipeSignature<t, $> {
    <a extends Morph<distillOut<t>>, r = instantiateType<inferPipes<t, [a]>, $>>(a: a): r;
    <a extends Morph<distillOut<t>>, b extends Morph<inferMorphOut<a>>, r = instantiateType<inferPipes<t, [a, b]>, $>>(a: a, b: b): r;
    <a extends Morph<distillOut<t>>, b extends Morph<inferMorphOut<a>>, c extends Morph<inferMorphOut<b>>, r = instantiateType<inferPipes<t, [a, b, c]>, $>>(a: a, b: b, c: c): r;
    <a extends Morph<distillOut<t>>, b extends Morph<inferMorphOut<a>>, c extends Morph<inferMorphOut<b>>, d extends Morph<inferMorphOut<c>>, r = instantiateType<inferPipes<t, [a, b, c, d]>, $>>(a: a, b: b, c: c, d: d): r;
    <a extends Morph<distillOut<t>>, b extends Morph<inferMorphOut<a>>, c extends Morph<inferMorphOut<b>>, d extends Morph<inferMorphOut<c>>, e extends Morph<inferMorphOut<d>>, r = instantiateType<inferPipes<t, [a, b, c, d, e]>, $>>(a: a, b: b, c: c, d: d, e: e): r;
    <a extends Morph<distillOut<t>>, b extends Morph<inferMorphOut<a>>, c extends Morph<inferMorphOut<b>>, d extends Morph<inferMorphOut<c>>, e extends Morph<inferMorphOut<d>>, f extends Morph<inferMorphOut<e>>, r = instantiateType<inferPipes<t, [a, b, c, d, e, f]>, $>>(a: a, b: b, c: c, d: d, e: e, f: f): r;
    <a extends Morph<distillOut<t>>, b extends Morph<inferMorphOut<a>>, c extends Morph<inferMorphOut<b>>, d extends Morph<inferMorphOut<c>>, e extends Morph<inferMorphOut<d>>, f extends Morph<inferMorphOut<e>>, g extends Morph<inferMorphOut<f>>, r = instantiateType<inferPipes<t, [a, b, c, d, e, f, g]>, $>>(a: a, b: b, c: c, d: d, e: e, f: f, g: g): r;
}
export interface ChainedPipes<t, $> extends ChainedPipeSignature<t, $> {
    try: ChainedPipeSignature<t, $>;
}
type validateChainedAsArgs<t> = [
    t
] extends [unset] ? [
    t
] extends [anyOrNever] ? [
] : [
    ErrorMessage<"as requires an explicit type parameter like myType.as<t>()">
] : [];
export type { Type as BaseType };
